#include <gcrypt.h>
#include <unistd.h>

static void
check (int rc)
{
  if (0 == rc)
    return;
  fprintf (stderr,
	   "Error: %d - %s\n",
	   rc,
	   gcry_strerror (rc));
  abort ();
}


int main()
{
  char key[256/8];
  char iv[96/8];
  char plaintext[1024];
  size_t plen;
  char ciphertext[sizeof (plaintext)];
  gcry_cipher_hd_t cipher;
  int rc;

  memset (key, 'A', sizeof (key));
  memset (iv, 'X', sizeof (iv));
  plen = 0;
  while (1)
  {
    ssize_t inlen = read (STDIN_FILENO,
			  &ciphertext[plen],
			  sizeof (ciphertext) - plen);
    if (-1 == inlen)
    {
      fprintf (stderr,
	       "Failed to read input\n");
      return 1;
    }
    if (0 == inlen)
      break;
    plen += inlen;
  }
  gcry_cipher_open (&cipher,
		    GCRY_CIPHER_AES256,
		    GCRY_CIPHER_MODE_GCM,
		    0);
  check (gcry_cipher_setkey (cipher,
			     key,
			     sizeof (key)));
  check (gcry_cipher_setiv (cipher,
			    iv,
			    sizeof (iv)));
  check (gcry_cipher_decrypt (cipher,			      
			      plaintext,
			      plen,
			      ciphertext,
			      plen));
  gcry_cipher_close (cipher);
  write (STDOUT_FILENO, plaintext, plen);
  return 0;
}
